#' Find marker genes using a binomial test implemented via `prop.test`
#'
#' This is intended to be a combination of the one-vs-rest methodology used in
#' Seurat and Scanpy with the binom testing option implemented in scran.
#'
#' @param sce A SingleCellExperiment object
#' @param pars parameters passed to the function
#'
#' @return A list containing the time of the computation and results of the
#'  method.
#'
run_binom <- function(sce, pars) {

  times <- system.time({

    n_groups <- length(unique(colLabels(sce)))
    n_genes <- nrow(sce)

    out <- vector("list", length = n_groups * n_genes)
    n <- 1
    for (i in seq_len(n_genes)) {

      gene <- rownames(sce)[[i]]
      y <- as.vector(counts(sce[i, ]))

      for (j in seq_len(n_groups)) {

        group <- levels(colLabels(sce))[[j]]

        x <- factor(as.numeric(colLabels(sce) == group))
        y_1 <- y[x == 0]
        y_2 <- y[x == 1]

        n_1 <- length(y_1)
        n_2 <- length(y_2)

        nz_1 <- sum(y_1 > 0)
        nz_2 <- sum(y_2 > 0)

        # FIXME: Hack!
        if (nz_1 == 0) {
          nz_1 <- nz_1 + 1
        }
        if (nz_2 == 0) {
          nz_2 <- nz_2 + 1
        }

        test <- prop.test(c(nz_1, nz_2), c(n_1, n_2))

        p_value <- test$p.value
        p_value_adj <- p_value / n_genes
        # FIXME: Is this sensible?
        t_value <- sqrt(test$statistic)
        score <- test$estimate[[1]]

        out[[n]] <- list(
          p_value = p_value,
          raw_statistic = score,
          p_value_adj = p_value_adj,
          gene = gene,
          cluster = group,
          scaled_statistic = t_value,
          log_fc = 0
        )
        n <- n + 1
      }
    }

    result <- dplyr::bind_rows(!!!out)
  })

  # Get the elapsed time.
  time <- times[[3]]

  validate_mgs_result(result)

  list(time = time, result = result, raw_result = result, pars = pars)
}
